import framebuf, utime, random, math
import logging
import threading

from utilities import *
from dink import eye, pupil
from Boxobs import *

class BASICxyOLED(): # all drawing objects need an x & y position, a width & height and the root "oled" object passed.
    ''' this is a root object that takes x, y & OLED as basic properties. All other objects can extend this'''
    def __init__(self, x, y, w, h, oled):
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.display = True # handy way of making this object invisible or not
        self.oled = oled
        return
    
    def setXY(self, newX, newY):
        self.x = newX
        self.y = newY
        
    def getXY(self):
        ''' simple [x,y] position return '''
        return([self.x, self.y])
        
    def onOff(self, TF):
        ''' send a True / False to switch this objects visibiliy '''
        self.display = TF    
    
class CIRCLE(BASICxyOLED):
    def __init__(self, x, y, w, h, oled, t): # xpos, ypos, width, height, oled, thickness    
        super().__init__(x, y, w, h, oled)
        self.r = int(w/2.0)
        self.t = t
        self.parent = self
        self.d = 0 # default distance (if an orbiting object)
        self.draw()
        
    def follow(self, who, r): # if this circle is part of an orbit, who does it follow?
        self.parent = who
        self.xy = self.parent.getHotspot()
        self.x = self.xy[0]
        self.y = self.xy[1]
        self.r = r  #r is passed from function calling follow (so needs to be calculated there)
        self.w = int(self.r*2)
        self.draw()
        #this worked before 14/02/21
        #self.parent = who
        #self.d = self.parent.getDefaultRadius()
        
    def setRadius(self, r):
        self.r = r
              
    def draw(self):
        if self.display:
            x, y, xx, yy = self.x-int(self.w/2), self.y-int(self.w/2), self.x+int(self.w/2), self.y+int(self.w/2)
            #self.oled.fill_rect(x,y,xx,yy, 0)
            for times in range(0,self.t):
                for d in range(0, 89):
                    rads = (d*4) * (math.pi/180.0)        
                    self.oled.pixel(self.x+int((self.r-times) * math.cos(rads)), self.y + int((self.r-times) * math.sin(rads)), 1)
                

class IM(BASICxyOLED):
    ims = []
    def showIMS():
        for i in IM.ims:
            i.draw()
    
    def connectIMS(imArray):
        pos = 0
        for i in imArray:
            i.moveMe()
            nextP = pos + 1
            if nextP < len(imArray) :
                imArray[0].oled.line(int(i.x), int(i.y), int(imArray[nextP].x), int(imArray[nextP].y), 1)
                pos+=1
                
    ''' Draws a hex image to the OLED '''    
    def __init__(self, x, y, w, h, oled, im): # width / height / HEX image from dink.py
        ''' IM needs width(w) height(h) the hexImage(im) and the Oled to draw to '''        
        super().__init__(x, y, w, h, oled) # take in the x,y,w,h,oled infor from the BASICxyOLED class
        self.tx = int(w/2) # target centre X (x position + 1/2 width)
        self.ty = int(h/2) # target centre Y (y position + 1/2 height)
        self.hx = self.tx # 'hotspots' default to centre 
        self.hy = self.ty
        self.im = framebuf.FrameBuffer(im, w, h, framebuf.MONO_HLSB) # make a frame buffer from the hex image
        self.vector = [0,0]
        IM.ims.append(self)
        
        self.draw()
        return
    
    def changeIm(self, newIm):
        self.onOff(True)
        self.im = framebuf.FrameBuffer(newIm, self.w, self.h, framebuf.MONO_HLSB) # make a frame buffer from the hex image
        self.draw()
            
    def setXY(self, xy):
        ''' change the position of the IM '''
        self.x = xy[0]
        self.y = xy[1]
        self.draw()        
        
    def setHotspot(self, newX, newY):
        ''' change the hotspot [w,h] of the IM '''
        self.hx = int(newX) # this new offset will be added to the X position in "getHotSpot()"
        self.hy = int(newY) # this new offset will be added to the Y position in "getHotSpot()"
    
    def getHotspot(self):
        ''' return the hotspot as an [x,y] array '''
        return([self.x + self.hx, self.y + self.hy])   
    
    def getRadius(self):
        if IM.connectIMS:
            # this is assuming the connectIMS [] has obejects in it (it shouldn't be called if not!)
            return(self.calculate_distance(IM.ims[1]))
        
    def calculate_distance(self, other_DOT):
        ''' calculate the pixel distance of this object to a target object '''
        return math.sqrt(
        ((self.x + self.hx) - other_DOT.getHotspot()[0])**2 +
        ((self.y + self.hy) - other_DOT.getHotspot()[1])**2)
    
    def setState(self, TF):        
        self.state = TF
        
    def setVector(self, v):
        ''' add a vector to this objects x,y position '''
        self.vector = v
    
    def moveMe(self):
        nextX = self.x + self.vector[0]
        nextY = self.y + self.vector[1]
        
        if nextX < 0 or nextX >= 128:
            self.vector[0]=self.vector[0]*-1            
        if nextY < 17 or nextY >= 64:
            self.vector[1]=self.vector[1]*-1
            
        self.x = nextX
        self.y = nextY
        
    def draw(self):
        ''' draw this object by blitting it (main loop needs to call oled.show() to render it) '''
        if self.display:
            self.oled.blit(self.im, int(self.x), int(self.y)) # draw the hex image at x,y

class EYE(BASICxyOLED):
    ''' Draws an EYE circle & Pupil image to the OLED '''
    eyes = []
    
    def watch(a,r):
        for e in EYE.eyes:
            e.look(a,r)
            
    def ping():
        print("ping from EYE class")
            
    def __init__(self, x, y, w, h, oled): # width / height / HEX image from dink.py
        ''' IM needs width(w) height(h) the hexImage(im) and the Oled to draw to '''        
        super().__init__(x, y, w, h, oled)
        self.eyeIm = IM(x, y, w, h, oled, eye)
        self.pupilIm = IM(self.eyeIm.tx, self.eyeIm.ty, 9, 9, oled, pupil) # eye tx,ty = centre
        EYE.eyes.append(self)
        
    def look(self, rotation, radius):
        rads = (rotation-90) * (math.pi/180.0)
        self.eyeIm.draw()
        self.pupilIm.setXY([int(radius *  math.cos(rads)) + self.pupilIm.tx - self.pupilIm.w, int(radius *  math.sin(rads)) + self.pupilIm.ty - self.pupilIm.h])
       
class POP():
    imWH = 9   
    
    def __init__(self, x, y, zzz, oled):
        self.x = x
        self.y = y
        self.c = 0
        self.onOff = False
        self.zzz = zzz
        self.oled = oled
        
    def getOn(self):
        return(self.onOff)
    
    def setXY(self, x, y):
        if not(self.onOff):
            self.x = x
            self.y = y
            self.popMe()
        
    def popMe(self):
        if self.c >= len(popSeq)-1:
            self.c = 0
            self.onOff = False
        else:
            self.onOff = True       
            self.c += 1
            self.draw()
            utime.sleep(self.zzz)
            self.draw()
            
    def draw(self):
        tempHexImage = framebuf.FrameBuffer(popSeq[self.c], POP.imWH, POP.imWH, framebuf.MONO_HLSB)
        self.oled.blit(tempHexImage, self.x - int(POP.imWH / 2), self.y - int(POP.imWH / 2)) # draw the hex image at x,y            
        #self.oled.show()
        
class DOT(): # 1x1 px "flea"
    dots = []
    
    def first():
        for d in DOT.dots:
            d.setState(False)
        DOT.dots[0].setState(True)
        DOT.showDots()
        
    def showDots():        
        for d in DOT.dots:
            d.draw()
            
    def cycleDots():
        DOT.dots = DOT.dots(DOT.dots.pop(0))
        DOT.showDots()
        
    def activate(x):
        allDots = DOT.dots
        offDots = allDots[:x] + allDots[x+1:]
        for off in offDots:
            off.setState(False)
        DOT.dots[x].setState(True)
        DOT.showDots()
    
    def __init__(self, x, y, oled):
        self.x = x
        self.y = y
        self.oled = oled
        self.w = 7
        self.state = False
        self.ims = [x7_0, x7_1] # hex images for not selected / selected dots
        DOT.dots.append(self)
        DOT.first() # reset all dots to False then set #1 to True (in DOT class)
        self.draw()
        
    def setState(self, TF):
        self.state = TF
        
    def switch(self):
        self.state = not(self.state)
        
    def getState(self):
        return(self.state)
    
    def calculate_distance(self, other_DOT):
        return math.sqrt(
        (self.x - other_DOT.x)**2 +
        (self.y - other_DOT.y)**2)
    
    def draw(self):
        myDotHex = framebuf.FrameBuffer(self.ims[self.state], self.w, self.w, framebuf.MONO_HLSB)
        self.oled.blit(myDotHex, self.x - int(self.w / 2), self.y - int(self.w / 2)) # draw the hex image from the centre          
        #self.oled.show()
        
class ORBIT_point(): # easist way to keep track of a point orbiting the ORBIT object?
    ''' an X,Y point that floats around the centre of the ORBIT object (parent) '''
    def __init__(self, parent, childNum):
        self.parent = parent
        self.childNum = childNum
        self.myAngOffset = self.parent.getAngleSpacing() * self.childNum
        self.x = 0
        self.y = 0
        self.addD = 0
        #print("I am node {} with an angleOffset of {} from my parent angle of {}".format(self.childNum, self.myAngOffset, self.parent.getRotation()))
        self.setXY()
        return
    
    def extendMyD(self, newD):
        self.addD = newD
        
    def retOrbitChildInfo(self):
        print("I am node {} with an angleOffset of {} from my parent angle of {}".format(self.childNum, self.myAngOffset, self.parent.getRotation()))
        
    def reshuffle(self):
        self.myAngOffset = self.parent.getAngleSpacing() * self.childNum
        return
        
    def setXY(self):        
        self.d = self.parent.getDefaultRadius() + self.addD
        self.a = self.parent.getRotation() + self.myAngOffset
        self.rads = self.a * (math.pi / 180)
        #print("i'm {} & my a = {}".format(self.childNum, self.a))
        pXY = self.parent.getXY()
        tempXY = [int(self.d *  math.cos(self.rads)), int(self.d *  math.sin(self.rads))] # no need to calculate an offset for centre of shapes as this is just a point [x,y]
        self.xy = [pXY[0]+ tempXY[0],pXY[1] + tempXY[1]]        
        self.x = self.xy[0]
        self.y = self.xy[1]
        return
        
    def getXY(self):
        return(self.xy)
    
                 
class ORBIT():
    orbits = []
    
    def info():
        for o in ORBIT.orbits:
            for child in o.childObs:
                child.retOrbitChildInfo()
                
    def __init__(self, oled, x=63, y=32, children=1, defaultRadius=5): # parent, [Children]
        self.oled = oled
        self.x = x
        self.y = y
        self.children = children
        self.followers = []
        self.spacer = self.getAngleSpacing()
        self.maxRadius = defaultRadius #set the maximum radius = the defaut radius
        self.radius = defaultRadius # set the current radius = the default fadius
        self.rotation = 0 # default rotation set to 0
        self.setChildObsArray()
        ORBIT.orbits.append(self)
        
    def setFollowers(self, followers):
        self.followers = followers
        
    def setChildObsArray(self):
        self.childObs = []
        for c in range(0, self.children):
            self.childObs.append(ORBIT_point(self, c))
        return
        
    def getChildObs(self):
        return(self.childObs)
    
    def setDefaultRadius(self, slideVal): # slider sends converted 0-32 px length (any more and it's offscreen)
        self.radius = round(mapXtoY(slideVal, 0, self.maxRadius, int(self.children*2.25), self.maxRadius), 0) # a tweak to make sure boxes don't overlap
            
        for c in self.childObs:
            c.setXY() # get the children to re-position themselves
        self.draw()
        if self.followers:
            for f in self.followers:
                f.draw()
        
    def getDefaultRadius(self):
        return(self.radius)
    
    def getAngleSpacing(self):
        #print(self.children)
        return(int(360/self.children))
    
    def reshuffle(self):
        ''' a change in orbiting children (+/-) so we need to re-configure them '''
        for n in range(0, self.children):            
                self.childObs[n].reshuffle()
                self.childObs[n].setXY()
                self.followers[n].follow(self.childObs[n])
                
    def changeNodes(self, addSubtract): # passes a root list of objects that follow orbit points
        ''' add or subtract a node '''
        
        if addSubtract == 1:
            if self.children <= 8:
                self.children+=1
                self.childObs.append(ORBIT_point(self, self.children))
                self.spacer = self.getAngleSpacing()
                self.setChildObsArray()
                self.followers.append(BOX(self.oled, 0, 0, 12, 12, 2, False, str(str(self.children))))
        else: # assuming a -1 is passed here!
            if self.children >=2:
                self.children-=1
                self.childObs.pop(len(self.childObs)-1)
                self.spacer = self.getAngleSpacing()
                self.setChildObsArray()
                self.followers.pop(self.children)
        
        self.reshuffle()
            
    def setRotation(self, potVal): # pot sends converted 0-359 angle (needs to be converted to rads later)
        self.rotation = potVal
        for c in self.childObs:
            c.setXY()
        self.draw()
        
    def getRotation(self):
        return(self.rotation)
                   
    def getXY(self):
        return([self.x, self.y])            
        
    def draw(self): # called from main loop to draw the orbit lines
        #we need to distribute the children evenly around the parent
        #print("hello from ORBIT draw my rotation is {} I have {} children spaced with {}°".format(self.rotation, self.children, self.spacer))
        for c in self.childObs:
            cXY = c.getXY()
            self.oled.line(self.x, self.y, cXY[0], cXY[1], 1) #draw a line from orbit centre to child point XY line weight 1px
        
    