# DINKY OSC development experiments by Philip Thompson
# for Digital Maker CIC www.digital-maker.co.uk
# financial support by Creative Scotland

from machine import Pin, I2C
from ssd1306 import SSD1306_I2C
import framebuf
import utime, string
import random
from dink import * # dinky's images as HEX arrays
from imOb import * # Image objects
from utilities import *

WIDTH  = 128                                            # oled display width
HEIGHT = 64                                             # oled display height

i2c = I2C(0)                                            # Init I2C using I2C0 defaults, SCL=Pin(GP9), SDA=Pin(GP8), freq=400000
print("I2C Address      : "+hex(i2c.scan()[0]).upper()) # Display device address
print("I2C Configuration: "+str(i2c))                   # Display I2C config

oled = SSD1306_I2C(WIDTH, HEIGHT, i2c)                  # Init oled display
oled.fill(0)
# Clear the oled display in case it has junk on it.
# Blit the image from the framebuffer to the oled display

pan = machine.ADC(26) # potentiometer in pin 26
slide = machine.ADC(27) # slideer in pin 27
but = Pin(0, Pin.IN, Pin.PULL_UP) #push button in pin 0

paw = IM(0,128, 19, 64, oled, paw_0) # make an IM object with the paw_0 hex from dink.py
paw.setHotspot(9, 58) # set the "hotspot" (centre) of the cat paw (adds the 2 values to the x, y position of the object

dots = 2 # change this for more 
numSQs = [] # blank list for holding IMs (letters that float around)
for i in range(0, dots): # loop to make IM objecs & add to the list "numsSQs"
    x = random.randint(5, 123) # generate a random x position (between 5 & 123px)
    y = random.randint(18, 59) # generate a random y position (between 18 px & 59px)
    numSQs.append(IM(x, y, 5,5,oled,fiveByfive.get(string.ascii_uppercase[i]))) # make an IM object 5x5 px with a letter image pulled form dinky.py
    numSQs[i].setVector([random.randint(-10, 10)/10.0, random.randint(-10, 10)/10.0,]) #apply a random vector to the IM object so it moves
    
score = 0 # score to keep track of how many letters you have collected
scoreIm = IM(123, 5, 5,5,oled,fiveByfive.get(str(score))) # render the score image in the top right

fCirc = CIRCLE(0, 0, 0, 0, oled, 1) # create a "CIRCLE" object that's used to plot the paw & "target"

myWordIM = [] # empty list to keep a distance plot from paw to target
def createWords(myWord):    
    posX = 0 #initialise a counter
    for l in myWord: #loop through how ever many chars are in "myWord"
        if l != " ": # if this char isn't a space....
            myWordIM.append(IM(5+(posX*6), 5, 5, 5, oled, fiveByfive.get(myWord[posX].upper()))) # create an IM object with the hex image of the current letter (uppercase) 
        posX+=1 # incriment the counter
        
createWords("000") # create a 3 char "word" to keep track of numbers for distance (max being 128px so we only need 3 chars to start with).

def distDisplay(val): # function to show the distance by using 3 hex images of numbers
    sV = str(int(val)) # convert the value of the distance to a string
    #print(sV)
    if len(numSQs)>=1: #if the target list is greater than 1
        if val < 10:    #if the distance is only 1 char long    
            myWordIM[0].changeIm(fiveByfive.get(sV[0])) #change the 1st char = the HEX image of the number
            myWordIM[1].onOff(False) #make 2nd char invisible
            myWordIM[2].onOff(False) #make 3rd char invisible
        elif val >= 10 and val < 100: # 2 chars (10 - 99)
            myWordIM[0].changeIm(fiveByfive.get(sV[0])) #change the 1st char = the HEX image of the 1st number in the string 
            myWordIM[1].changeIm(fiveByfive.get(sV[1])) #change the 2nd char = the HEX image of the  2nd number
            myWordIM[2].onOff(False) # make 3rd char invisible
        else:
            myWordIM[0].changeIm(fiveByfive.get(sV[0])) #change the 1st char
            myWordIM[1].changeIm(fiveByfive.get(sV[1])) #change the 2nd char
            myWordIM[2].changeIm(fiveByfive.get(sV[2])) #change the 3rd char
    else: # stop displaying the distance
        myWordIM[0].onOff(False)
        myWordIM[1].onOff(False)
        myWordIM[2].onOff(False)
    
t1 = utime.ticks_ms() # set a t1 (miliseconds)
shots = 0 # for counting the shots taken.
gameOn = True # so we can "end the game"

while gameOn:    
    oled.fill(0) # blank screen
    
    x = int(round(mapXtoY(pan.read_u16(), 0, 65535, 128, 0), 0)) # x val of paw generated from pot (mapped to 0 - 128px)
    y = int(round(mapXtoY(slide.read_u16(), 0, 65535, 0, -64), 0)) # y val of paw generated from slider( -64 - 0px) 
    #
    if len(numSQs) > 0: # still targets floating around
        IM.connectIMS(numSQs) # cycle through the IM object "ims" list & draw a line from 1st to 2nd etc
        IM.showIMS() # all IM objects get told to show
    else: # grabbed all the targets, stop the game!
        t2 = utime.ticks_ms() # get the end time (miliseconds)
        #display the score / shots taken / time taken 
        oled.text("Got {} targets".format(dots), 10, 17)
        oled.text("with {} shots".format(shots), 10, 27)
        oled.text("in {} secs".format(round((t2-t1)/1000.0), 1), 10, 37)        
        oled.show()    # render the text
        gameOn = False # stop the game
            
    if len(numSQs)>0: #if the  numSQs IM list has IM obejcts in it...
        paw.setXY([x,y]) # draw the paw at x,y
        hotspot = paw.getHotspot() # reset the
        dist = paw.calculate_distance(numSQs[0]) # calculate the distance between the paw & the 1st in the list ("target dot")
        fCirc.follow(paw, dist) # draw a circle that has the paw as the centre & the radius is defined by the "target" dot
        distDisplay(dist) # call the function to show the distance in the top left with the 3 IMs showing number HEX images
    else:
        paw.onOff(False) # all targets caught, we can make the paw invisible now
    
    if but.value() == 0:# button press detected
        shots+=1
        if len(numSQs)>0: #if the target list is greater than 1 then we can "pop" targets
            paw.changeIm(paw_0i) # change the HEX image to be the filled in paw
            
            if dist <= 10: # if the paw's hotspet is within 10px of the target...
                #print("pop! {}".format(dist)) # feedback to the repl
                numSQs[0].onOff(False) # make the IM object (taget) invisible
                if len(numSQs)>0: # if the IM target list has objects in it
                    score+=1 # incriment the score by 1
                    scoreIm.changeIm(fiveByfive.get(str(score))) # render the new score
                    numSQs.pop(0) # pop the target out of the numSQs list
            else:                
                print("miss! {}".format(dist))
            utime.sleep(0.2) # small sleep to avoid button press multiple hits
            paw.changeIm(paw_0)
        
    oled.show() # render all the drawn objects to the oled
